//---------------------------------------------------------------------------

#ifndef ArmScopeH
#define ArmScopeH
//---------------------------------------------------------------------------

#include "..\miniscope\device\MiniscopeDeviceCapabilities.h"
#include "..\miniscope\device\MiniscopeDevice.h"
#include "ThreadComm.h"
#include "Decimator.h"
#include <deque>

class ArmScope
{
private:
	enum { VENDOR_ID = 0x03EB };
	enum { PRODUCT_ID = 0x6130 };

	bool bDeviceOpened;
	bool bStopRequest;
	ThreadComm *threadComm;

	enum E_CONNECTION_STATE conn_state;
	unsigned char* hdr;
	unsigned int hdr_bytes;	///< skipped reply header bytes
	struct S_SCOPE_SETTINGS settings, upcoming_settings;
	bool triggered;		///< we should start sending data buffer now
	bool bManualTrigger;///< manual trigger request flag
	unsigned int frame_position;	///< current data buffer position (0 when just sent trigger signal)

    Decimator decimator;

	std::deque<char> deque_trigger;	///< samples buffer for triggering
	short samples[ThreadComm::E_USB_FRAME_SIZE];
	short output_buffer[32768];
	/** \brief Test for trigger condition
		\param len number of samples added to trigger queue recently
		\return index of first sample after triggering, < 0 if not triggered
	*/
	int TestTrigger(int len);

	typedef void (*CallbackLog)(char*);
	typedef void (*CallbackData)(short*, int);
	typedef void (*CallbackConnect)(int, char*);
	typedef void (*CallbackTrigger)(void);
	void log(char *lpData, ...);
	void SetConnectionState(enum E_CONNECTION_STATE state, const char* const descr);

public:
	ArmScope();
	~ArmScope();
	int Connect(void);
	int Disconnect(void);
	int RecStart(void);				///< start recording
	int RecStop(void);              ///< stop recording
	CallbackLog callbackLog;
	CallbackData callbackData;
	CallbackConnect callbackConnect;
	CallbackTrigger callbackTrigger;
	void DataRcvCb(unsigned char data[], unsigned int len);
	/** \brief Request manual triggering
		\return 0 on success
	*/
	int ManualTrigger(void);
	int SetTriggerType(enum E_TRIGGER_TYPE);
	int SetCouplingType(enum E_COUPLING_TYPE);
	/** \brief Set capture buffer size
		\param size number of samples
	*/
	int SetBufferSize(int size);
	int SetSensitivity(int id);
	int SetSampling(int id);
	int SetTriggerLevel(int level);
};
extern ArmScope armScope;


#endif
